# provision the apps
resource "helm_release" "app" {
  # provision only those apps which were enabled explicitly
  for_each = { for k, v in local.expanded_apps_settings : k => v if lookup(v, "enabled", false) }

  name             = each.key
  chart            = each.value["chart"]
  repository       = each.value["repository"]
  version          = each.value["version"]
  namespace        = each.value["namespace"]
  create_namespace = true
  timeout = 1000
  
  max_history = each.value["max_history"]

  values = lookup(each.value["values"], "file", [])

  dynamic "set" {
    for_each = each.value["values"]["set"]

    content {
      name  = set.key
      value = set.value
    }
  }
}

# since the istio is still not in the helm repo
# we will need to download the release and install it from the local filesystem
resource "null_resource" "istio" {
  count = lookup(var.istio, "enabled", false) ? 1 : 0

  triggers = {
    always_run = timestamp()
  }

  provisioner "local-exec" {
    command = "apk add curl && curl -L https://istio.io/downloadIstio | ISTIO_VERSION=${var.istio["version"]} sh -"
  }
}

resource "helm_release" "istio_base" {
  count = lookup(var.istio, "enabled", false) ? 1 : 0

  name             = "istio-base"
  chart            = "./istio-${var.istio["version"]}/manifests/charts/base"
  description      = "Istio version: ${var.istio["version"]}"
  namespace        = "istio-system"
  create_namespace = true

  max_history = 3

  depends_on = [null_resource.istio]
}

resource "helm_release" "istiod" {
  count = lookup(var.istio, "enabled", false) ? 1 : 0

  name             = "istiod"
  chart            = "./istio-${var.istio["version"]}/manifests/charts/istio-control/istio-discovery"
  description      = "Istio version: ${var.istio["version"]}"
  namespace        = "istio-system"
  create_namespace = true

  max_history = 3

  set {
    name  = "global.hub"
    value = "docker.io/istio"
  }

  set {
    name  = "global.tag"
    value = var.istio["version"]
  }

  # aditional values for discovery
  dynamic "set" {
    for_each = lookup(var.istio, "set", {})

    content {
      name  = each.key
      value = each.value
    }
  }

  depends_on = [null_resource.istio, helm_release.istio_base]
}

# remove the downloaded artifacts
resource "null_resource" "istio_cleanup" {
  count = lookup(var.istio, "enabled", false) ? 1 : 0

  triggers = {
    always_run = timestamp()
  }

  provisioner "local-exec" {
    command = "[ -d './istio-${var.istio["version"]}' ] && (rm -rf  './istio-${var.istio["version"]}' || true)"
  }

  depends_on = [helm_release.istio_base, helm_release.istiod, null_resource.istio]
}
